from __future__ import unicode_literals

import os

from mopidy import config, ext

from .touch_screen import TouchScreen


__version__ = '1.0.0'


class Extension(ext.Extension):
    dist_name = 'Mopidy-Touchscreen'
    ext_name = 'touchscreen'
    version = __version__

    def get_default_config(self):
        conf_file = os.path.join(os.path.dirname(__file__),
                                 'ext.conf')
        return config.read(conf_file)

    def get_config_schema(self):
        schema = super(Extension, self).get_config_schema()
        schema['screen_width'] = config.Integer(minimum=1)
        schema['screen_height'] = config.Integer(minimum=1)
        schema['resolution_factor'] = config.Integer(minimum=6)
        schema['cursor'] = config.Boolean()
        schema['fullscreen'] = config.Boolean()
        schema['cache_dir'] = config.Path()
        schema['gpio'] = config.Boolean()
        schema['gpio_left'] = config.Integer()
        schema['gpio_right'] = config.Integer()
        schema['gpio_up'] = config.Integer()
        schema['gpio_down'] = config.Integer()
        schema['gpio_enter'] = config.Integer()
        schema['sdl_fbdev'] = config.String()
        schema['sdl_mousdrv'] = config.String()
        schema['sdl_mousedev'] = config.String()
        schema['sdl_audiodriver'] = config.String()
        schema['sdl_path_dsp'] = config.String()
        return schema

    def setup(self, registry):
        registry.add('frontend', TouchScreen)



import pygame

change_speed = 2


class DynamicBackground:

    def __init__(self, size):
        self.image_loaded = False
        self.size = size
        self.surface = pygame.Surface(self.size).convert()
        self.surface.fill((145, 16, 16))
        self.surface_image = pygame.Surface(self.size).convert()
        self.update = True

    def draw_background(self):
        if self.image_loaded:
            return self.surface_image.copy()
        else:
            return self.surface.copy()

    def should_update(self):
        if self.update:
            self.update = False
            return True
        else:
            return False

    def set_background_image(self, image):
        if image is not None:
            image_size = get_aspect_scale_size(image, self.size)
            target = pygame.transform.smoothscale(image, image_size)
            target.set_alpha(150)
            self.image_loaded = True
            self.surface_image.fill((0, 0, 0))
            pos = ((self.size[0] - image_size[0])/2,
                   (self.size[1] - image_size[1])/2)
            self.surface_image.blit(target, pos)
        else:
            self.image_loaded = False
        self.update = True


def get_aspect_scale_size(img, new_size):
    size = img.get_size()
    aspect_x = new_size[0] / float(size[0])
    aspect_y = new_size[1] / float(size[1])
    if aspect_x > aspect_y:
        aspect = aspect_x
    else:
        aspect = aspect_y
    new_size = (int(aspect*size[0]), int(aspect*size[1]))
    return new_size



import logging

import RPi.GPIO as GPIO

import pygame


logger = logging.getLogger(__name__)


class GPIOManager():
    def __init__(self, pins):
        GPIO.setmode(GPIO.BCM)

        # Left Button
        GPIO.setup(pins['left'], GPIO.IN, pull_up_down=GPIO.PUD_UP)
        GPIO.add_event_detect(pins['left'], GPIO.BOTH, callback=left,
                              bouncetime=30)

        # Right Button
        GPIO.setup(pins['right'], GPIO.IN, pull_up_down=GPIO.PUD_UP)
        GPIO.add_event_detect(pins['right'], GPIO.BOTH,
                              callback=right,
                              bouncetime=30)

        # Up Button
        GPIO.setup(pins['up'], GPIO.IN, pull_up_down=GPIO.PUD_UP)
        GPIO.add_event_detect(pins['up'], GPIO.BOTH, callback=up,
                              bouncetime=30)

        # Down Button
        GPIO.setup(pins['down'], GPIO.IN, pull_up_down=GPIO.PUD_UP)
        GPIO.add_event_detect(pins['down'], GPIO.BOTH, callback=right,
                              bouncetime=30)

        # Enter Button
        GPIO.setup(pins['enter'], GPIO.IN, pull_up_down=GPIO.PUD_UP)
        GPIO.add_event_detect(pins['enter'], GPIO.BOTH,
                              callback=right,
                              bouncetime=30)


def right(channel):
    dict = {}
    if GPIO.input(channel) == 1:
        type = pygame.KEYUP
    else:
        type = pygame.KEYDOWN
        dict['unicode'] = None
    dict['key'] = pygame.K_RIGHT
    event = pygame.event.Event(type, dict)
    pygame.event.post(event)


def left(channel):
    dict = {}
    if GPIO.input(channel) == 1:
        type = pygame.KEYUP
    else:
        type = pygame.KEYDOWN
        dict['unicode'] = None
    dict['key'] = pygame.K_RIGHT
    event = pygame.event.Event(type, dict)
    pygame.event.post(event)


def down(channel):
    dict = {}
    if GPIO.input(channel) == 1:
        type = pygame.KEYUP
    else:
        type = pygame.KEYDOWN
        dict['unicode'] = None
    dict['key'] = pygame.K_DOWN
    event = pygame.event.Event(type, dict)
    pygame.event.post(event)


def up(channel):
    dict = {}
    if GPIO.input(channel) == 1:
        type = pygame.KEYUP
    else:
        type = pygame.KEYDOWN
        dict['unicode'] = None
    dict['key'] = pygame.K_UP
    event = pygame.event.Event(type, dict)
    pygame.event.post(event)


def enter(channel):
    dict = {}
    if GPIO.input(channel) == 1:
        type = pygame.KEYUP
    else:
        type = pygame.KEYDOWN
        dict['unicode'] = None
    dict['key'] = pygame.K_RETURN
    event = pygame.event.Event(type, dict)
    pygame.event.post(event)



class BaseScreen():

    update_all = 0
    update_partial = 1
    no_update = 2

    def __init__(self, size, base_size, manager, fonts):
        self.size = size
        self.base_size = base_size
        self.manager = manager
        self.fonts = fonts

    def update(self, surface, update_type, rects):
        """
        Draw this screen to the surface

        :param surface:
        :param update_type:
        :param rects:
        """
        pass

    def event(self, event):
        pass

    def change_screen(self, direction):
        return False

    def should_update(self):
        return BaseScreen.update_partial



import pygame

from .base_screen import BaseScreen
from ..graphic_utils import ScreenObjectsManager, TouchAndTextItem
from ..input import InputManager


class Keyboard(BaseScreen):

    def __init__(self, size, base_size, manager, fonts, listener):
        BaseScreen.__init__(self, size, base_size, manager, fonts)
        self.base_width = size[0]/10
        self.base_height = size[1]/5
        self.listener = listener
        self.manager = manager
        self.selected_row = 0
        self.selected_col = 0
        self.selected_others = -1
        self.font = pygame.font.SysFont("arial", size[1]/6)
        self.keyboards = [ScreenObjectsManager(), ScreenObjectsManager()]
        self.other_objects = ScreenObjectsManager()
        self.current_keyboard = 0

        self.keys = [[['q', 'w', 'e', 'r', 't', 'y', 'u', 'i', 'o', 'p'],
                      ['a', 's', 'd', 'f', 'g', 'h', 'j', 'k', 'l', '-'],
                      [',', 'z', 'x', 'c', 'v', 'b', 'n', 'm', '.', '_']],

                     [['1', '2', '3', '4', '5', '6', '7', '8', '9', '0'],
                      ['!', '@', '#', '$', '%', '&', '/', '(', ')', '='],
                      ['?', '{', '}', '_', '[', ']', '+', '<', '>', '*']]]

        line = self.base_height
        for row in self.keys[self.current_keyboard]:
            pos = 0
            for key in row:
                button = \
                    TouchAndTextItem(self.font, key,
                                     (pos, line),
                                     (self.base_width, self.base_height),
                                     center=True, background=(150, 150, 150))
                self.keyboards[self.current_keyboard].\
                    set_touch_object(key, button)
                pos += self.base_width
            line += self.base_height

        self.current_keyboard = 1
        line = self.base_height
        for row in self.keys[self.current_keyboard]:
            pos = 0
            for key in row:
                button = \
                    TouchAndTextItem(self.font, key, (pos, line),
                                     (self.base_width, self.base_height),
                                     center=True, background=(150, 150, 150),
                                     scroll_no_fit=False)
                self.keyboards[self.current_keyboard].\
                    set_touch_object(key, button)
                pos += self.base_width
            line += self.base_height
        self.current_keyboard = 0

        # Symbol button
        button = TouchAndTextItem(self.font, "123",
                                  (0, self.base_height*4),
                                  (self.base_width*2, self.base_height),
                                  center=True, background=(150, 150, 150),
                                  scroll_no_fit=False)
        self.other_objects.set_touch_object("symbols", button)

        # remove button
        button = TouchAndTextItem(self.font, "<-",
                                  (self.base_width*2, self.base_height*4),
                                  (self.base_width*2, self.base_height),
                                  center=True, background=(150, 150, 150),
                                  scroll_no_fit=False)
        self.other_objects.set_touch_object("remove", button)

        # Space button
        button = TouchAndTextItem(self.font, " ",
                                  (self.base_width*4, self.base_height*4),
                                  (self.base_width*4, self.base_height),
                                  center=True, background=(150, 150, 150),
                                  scroll_no_fit=False)
        self.other_objects.set_touch_object("space", button)

        # OK button
        button = TouchAndTextItem(self.font, "->",
                                  (self.base_width*8, self.base_height*4),
                                  (self.base_width*2, self.base_height),
                                  center=True, background=(150, 150, 150),
                                  scroll_no_fit=False)
        self.other_objects.set_touch_object("ok", button)

        # EditText button
        button = TouchAndTextItem(self.font, "",
                                  (0, 0),
                                  (self.size[0], self.base_height),
                                  center=False, scroll_no_fit=False)
        self.other_objects.set_object("text", button)
        self.change_selected(0, 0)

    def update(self, screen):
        screen.fill((0, 0, 0))
        self.keyboards[self.current_keyboard].render(screen)
        self.other_objects.render(screen)

    def touch_event(self, touch_event):
        if touch_event.type == InputManager.click:
            keys = self.keyboards[self.current_keyboard]\
                .get_touch_objects_in_pos(touch_event.current_pos)
            for key in keys:
                self.other_objects.get_object("text").add_text(key, False)
            keys = self.other_objects.get_touch_objects_in_pos(
                touch_event.current_pos)
            for key in keys:
                if key == 'symbols':
                    self.change_keyboard()
                elif key == "remove":
                    self.other_objects.get_object("text").remove_text(1, False)
                elif key == "space":
                    self.other_objects.get_object("text").add_text(" ", False)
                elif key == "ok":
                    text = self.other_objects.get_object("text").text
                    self.listener.text_input(text)
                    self.manager.close_keyboard()
        elif touch_event.type == InputManager.key:
            if not isinstance(touch_event.unicode, int):
                if touch_event.unicode == u'\x08':
                    self.other_objects.get_object("text").remove_text(1, False)
                else:
                    self.other_objects.get_object("text").add_text(
                        touch_event.unicode, False)
            elif touch_event.direction is not None:
                x = 0
                y = 0
                if touch_event.direction == InputManager.left:
                    x = -1
                elif touch_event.direction == InputManager.right:
                    x = 1
                elif touch_event.direction == InputManager.up:
                    y = -1
                elif touch_event.direction == InputManager.down:
                    y = 1
                if touch_event.direction == InputManager.enter:
                    self.selected_click()
                else:
                    self.change_selected(x, y)

    def change_keyboard(self):
        if self.current_keyboard == 0:
            self.current_keyboard = 1
        else:
            self.current_keyboard = 0
        if self.selected_others < 0:
            self.change_selected(0, 0)

    def change_selected(self, x, y):
        if self.selected_others < 0:
            # We are on the keyboard

            # Update col
            self.selected_col += x
            if self.selected_col < 0:
                self.selected_col = 0
            elif self.selected_col > 9:
                self.selected_col = 9

            # Update row
            self.selected_row += y
            if self.selected_row < 0:
                self.selected_row = 0
            elif self.selected_row > 2:

                # Change to the bottom part
                if self.selected_col < 2:
                    self.selected_others = 0
                elif self.selected_col < 4:
                    self.selected_others = 1
                elif self.selected_col < 8:
                    self.selected_others = 2
                else:
                    self.selected_others = 3

            # Update selected
            if self.selected_others < 0:
                key = self.keys[self.current_keyboard][
                    self.selected_row][self.selected_col]
                self.keyboards[self.current_keyboard].set_selected(key)
            else:
                self.keyboards[0].set_selected(None)
                self.keyboards[1].set_selected(None)
                self.set_selected_other()
        else:
            # We are on the bottom part
            if y < 0:
                # we are returning to the keyboard

                # Select col
                if self.selected_others == 0:
                    self.selected_col = 0
                elif self.selected_others == 1:
                    self.selected_col = 2
                elif self.selected_others == 2:
                    self.selected_col = 4
                else:
                    self.selected_col = 8

                self.selected_others = -1
                self.set_selected_other()

                self.selected_row = 2
                key = self.keys[self.current_keyboard][
                    self.selected_row][self.selected_col]
                self.keyboards[self.current_keyboard].set_selected(key)
            elif x != 0:
                # We change in horizontal
                self.selected_others += x
                if self.selected_others < 0:
                    self.selected_others = 0
                elif self.selected_others > 3:
                    self.selected_others = 3
                self.set_selected_other()

    def selected_click(self):
        if self.selected_others >= 0:
            if self.selected_others == 0:
                self.change_keyboard()
            elif self.selected_others == 1:
                self.other_objects.get_object("text").remove_text(1, False)
            elif self.selected_others == 2:
                self.other_objects.get_object("text").add_text(" ", False)
            elif self.selected_others == 3:
                text = self.other_objects.get_object("text").text
                self.listener.text_input(text)
                self.manager.close_keyboard()
        else:
            key = self.keys[self.current_keyboard][
                self.selected_row][self.selected_col]
            self.other_objects.get_object("text").add_text(
                key, False)

    def set_selected_other(self):
        key = None
        if self.selected_others == 0:
            key = "symbols"
        elif self.selected_others == 1:
            key = "remove"
        elif self.selected_others == 2:
            key = "space"
        elif self.selected_others == 3:
            key = "ok"
        self.other_objects.set_selected(key)



import logging
import os
import traceback
from threading import Thread

from mopidy import core, exceptions

import pygame

import pykka

from .screen_manager import ScreenManager


logger = logging.getLogger(__name__)


class TouchScreen(pykka.ThreadingActor, core.CoreListener):
    def __init__(self, config, core):
        super(TouchScreen, self).__init__()
        self.core = core
        self.running = False
        self.cursor = config['touchscreen']['cursor']
        self.cache_dir = config['touchscreen']['cache_dir']
        self.fullscreen = config['touchscreen']['fullscreen']
        self.screen_size = (config['touchscreen']['screen_width'],
                            config['touchscreen']['screen_height'])
        self.resolution_factor = (config['touchscreen']['resolution_factor'])
        if config['touchscreen']['sdl_fbdev'].lower() != "none":
            os.environ["SDL_FBDEV"] = config['touchscreen']['sdl_fbdev']
        if config['touchscreen']['sdl_mousdrv'].lower() != "none":
            os.environ["SDL_MOUSEDRV"] = (
                config['touchscreen']['sdl_mousdrv'])

        if config['touchscreen']['sdl_mousedev'].lower() != "none":
            os.environ["SDL_MOUSEDEV"] = config['touchscreen']['sdl_mousedev']

        if config['touchscreen']['sdl_audiodriver'].lower() != "none":
            os.environ["SDL_AUDIODRIVER"] = (
                config['touchscreen']['sdl_audiodriver'])

        os.environ["SDL_PATH_DSP"] = config['touchscreen']['sdl_path_dsp']
        pygame.init()
        pygame.display.set_caption("Mopidy-Touchscreen")
        self.get_display_surface(self.screen_size)
        pygame.mouse.set_visible(self.cursor)
        self.screen_manager = ScreenManager(self.screen_size,
                                            self.core,
                                            self.cache_dir,
                                            self.resolution_factor)

        # Raspberry pi GPIO
        self.gpio = config['touchscreen']['gpio']
        if self.gpio:

            from .input import GPIOManager

            pins = {}
            pins['left'] = config['touchscreen']['gpio_left']
            pins['right'] = config['touchscreen']['gpio_right']
            pins['up'] = config['touchscreen']['gpio_up']
            pins['down'] = config['touchscreen']['gpio_down']
            pins['enter'] = config['touchscreen']['gpio_enter']
            self.gpio_manager = GPIOManager(pins)

    def get_display_surface(self, size):
        try:
            if self.fullscreen:
                self.screen = pygame.display.set_mode(
                    size, pygame.FULLSCREEN)
            else:
                self.screen = pygame.display.set_mode(size, pygame.RESIZABLE)
        except Exception:
            raise exceptions.FrontendError("Error on display init:\n"
                                           + traceback.format_exc())

    def start_thread(self):
        clock = pygame.time.Clock()
        pygame.event.set_blocked(pygame.MOUSEMOTION)
        while self.running:
            clock.tick(12)
            self.screen_manager.update(self.screen)
            for event in pygame.event.get():
                if event.type == pygame.QUIT:
                    os.system("pkill mopidy")
                elif event.type == pygame.VIDEORESIZE:
                    self.get_display_surface(event.size)
                    self.screen_manager.resize(event)
                else:
                    self.screen_manager.event(event)
        pygame.quit()

    def on_start(self):
        try:
            self.running = True
            thread = Thread(target=self.start_thread)
            thread.start()
        except:
            traceback.print_exc()

    def on_stop(self):
        self.running = False

    def track_playback_started(self, tl_track):
        try:
            self.screen_manager.track_started(tl_track)
        except:
            traceback.print_exc()

    def volume_changed(self, volume):
        self.screen_manager.volume_changed(volume)

    def playback_state_changed(self, old_state, new_state):
        self.screen_manager.playback_state_changed(old_state,
                                                   new_state)

    def tracklist_changed(self):
        try:
            self.screen_manager.tracklist_changed()
        except:
            traceback.print_exc()

    def track_playback_ended(self, tl_track, time_position):
        self.screen_manager.track_playback_ended(tl_track,
                                                 time_position)

    def options_changed(self):
        try:
            self.screen_manager.options_changed()
        except:
            traceback.print_exc()

    def playlists_loaded(self):
        self.screen_manager.playlists_loaded()



from base_screen import BaseScreen

import mopidy.models

from ..graphic_utils import ListView


class LibraryScreen(BaseScreen):
    def __init__(self, size, base_size, manager, fonts):
        BaseScreen.__init__(self, size, base_size, manager, fonts)
        self.list_view = ListView((0, 0), (
            self.size[0], self.size[1] -
            self.base_size), self.base_size, self.fonts['base'])
        self.directory_list = []
        self.current_directory = None
        self.library = None
        self.library_strings = None
        self.browse_uri(None)

    def go_inside_directory(self, uri):
        self.directory_list.append(self.current_directory)
        self.current_directory = uri
        self.browse_uri(uri)

    def browse_uri(self, uri):
        self.library_strings = []
        if uri is not None:
            self.library_strings.append("../")
        self.library = self.manager.core.library.browse(uri).get()
        for lib in self.library:
            self.library_strings.append(lib.name)
        self.list_view.set_list(self.library_strings)

    def go_up_directory(self):
        if len(self.directory_list):
            directory = self.directory_list.pop()
            self.current_directory = directory
            self.browse_uri(directory)

    def should_update(self):
        return self.list_view.should_update()

    def update(self, screen, update_type, rects):
        update_all = (update_type == BaseScreen.update_all)
        self.list_view.render(screen, update_all, rects)

    def touch_event(self, touch_event):
        clicked = self.list_view.touch_event(touch_event)
        if clicked is not None:
            if self.current_directory is not None:
                if clicked == 0:
                    self.go_up_directory()
                else:
                    if self.library[clicked - 1].type\
                            == mopidy.models.Ref.TRACK:
                        self.play_uri(clicked-1)
                    else:
                        self.go_inside_directory(
                            self.library[clicked - 1].uri)
            else:
                self.go_inside_directory(self.library[clicked].uri)

    def play_uri(self, track_pos):
        self.manager.core.tracklist.clear()
        tracks = []
        for item in self.library:
            if item.type == mopidy.models.Ref.TRACK:
                tracks.append(self.manager.core.library.lookup(
                    item.uri).get()[0])
            else:
                track_pos -= 1
        self.manager.core.tracklist.add(tracks)
        self.manager.core.playback.play(
            tl_track=self.manager.core.tracklist.tl_tracks.get()
            [track_pos])



import logging

from screen_objects import ScreenObjectsManager, ScrollBar, \
    TouchAndTextItem

from ..input import InputManager

logger = logging.getLogger(__name__)


class ListView():
    def __init__(self, pos, size, base_size, font):
        self.size = size
        self.pos = pos
        self.base_size = base_size
        self.screen_objects = ScreenObjectsManager()
        self.max_rows = self.size[1] / self.base_size
        self.current_item = 0
        self.font = font
        self.list_size = 0
        self.list = []
        self.scrollbar = False
        self.selected = None
        self.active = []
        self.set_list([])
        self.update_keys = []

    # Sets the list for the lisview.
    # It should be an iterable of strings
    def set_list(self, item_list):
        self.screen_objects.clear()
        self.list = item_list
        self.list_size = len(item_list)
        if self.max_rows < self.list_size:
            self.scrollbar = True
            scroll_bar = ScrollBar(
                (self.pos[0] + self.size[0] - self.base_size,
                 self.pos[1]),
                (self.base_size, self.size[1]), self.list_size,
                self.max_rows)
            self.screen_objects.set_touch_object("scrollbar",
                                                 scroll_bar)
        else:
            self.scrollbar = False
        if self.list_size > 0:
            self.selected = 0
        else:
            self.selected = None
        self.load_new_item_position(0)

    # Will load items currently displaying in item_pos
    def load_new_item_position(self, item_pos):
        self.update_keys = []
        self.current_item = item_pos
        if self.scrollbar:
            self.screen_objects.clear_touch(["scrollbar"])
        else:
            self.screen_objects.clear_touch(None)
        i = self.current_item
        z = 0
        if self.scrollbar:
            width = self.size[0] - self.base_size
        else:
            width = self.size[0]
        self.should_update_always = False
        while i < self.list_size and z < self.max_rows:
            item = TouchAndTextItem(self.font, self.list[i], (
                self.pos[0],
                self.pos[1] + self.base_size * z), (width, -1))
            if not item.fit_horizontal:
                self.update_keys.append(str(i))
            self.screen_objects.set_touch_object(str(i), item)
            i += 1
            z += 1
        self.reload_selected()

    def should_update(self):
        if len(self.update_keys) > 0:
            return True
        else:
            return False

    def render(self, surface, update_all, rects):
        if update_all:
            self.screen_objects.render(surface)
        else:
            for key in self.update_keys:
                object = self.screen_objects.get_touch_object(key)
                object.update()
                object.render(surface)
                rects.append(object.rect_in_pos)

    def touch_event(self, touch_event):
        self.must_update = True
        if touch_event.type == InputManager.click \
                or touch_event.type == InputManager.long_click:
            objects = self.screen_objects.get_touch_objects_in_pos(
                touch_event.current_pos)
            if objects is not None:
                for key in objects:
                    if key == "scrollbar":
                        direction = \
                            self.screen_objects.get_touch_object(
                                key).touch(touch_event.current_pos)
                        if direction != 0:
                            self.move_to(direction)
                    else:
                        return int(key)
        elif (touch_event.type == InputManager.key and
                self.selected is not None):
            if touch_event.direction == InputManager.enter:
                if self.selected is not None:
                    return self.selected
            elif touch_event.direction == InputManager.up:
                self.set_selected(self.selected-1)
            elif touch_event.direction == InputManager.down:
                self.set_selected(self.selected+1)
        elif touch_event.type == InputManager.swipe:
            if touch_event.direction == InputManager.up:
                self.move_to(-1)
            elif touch_event.direction == InputManager.down:
                self.move_to(1)

    # Scroll to direction
    # direction == 1 will scroll down
    # direction == -1 will scroll up
    def move_to(self, direction):
        if self.scrollbar:
            if direction == 1:
                self.current_item += self.max_rows
                if self.current_item + self.max_rows > self.list_size:
                    self.current_item = self.list_size - self.max_rows
                self.load_new_item_position(self.current_item)
                self.screen_objects.get_touch_object(
                    "scrollbar").set_item(
                    self.current_item)
            elif direction == -1:
                self.current_item -= self.max_rows
                if self.current_item < 0:
                    self.current_item = 0
                self.load_new_item_position(self.current_item)
                self.screen_objects.get_touch_object(
                    "scrollbar").set_item(
                    self.current_item)
            self.set_active(self.active)

    # Set active items
    def set_active(self, active):
        self.must_update = True
        for number in self.active:
            try:
                self.screen_objects.get_touch_object(
                    str(number)).set_active(
                    False)
            except KeyError:
                pass
        for number in active:
            try:
                self.screen_objects.get_touch_object(
                    str(number)).set_active(
                    True)
            except KeyError:
                pass
        self.active = active

    def set_selected(self, selected):
        self.must_update = True
        if selected > -1 and selected < len(self.list):
            if self.selected is not None:
                try:
                    self.screen_objects.get_touch_object(
                        str(self.selected)).set_selected(
                        False)
                except KeyError:
                    pass
            if selected is not None:
                try:
                    self.screen_objects.get_touch_object(
                        str(selected)).set_selected(
                        True)
                except KeyError:
                    pass
            self.selected = selected
            self.set_selected_on_screen()

    def set_selected_on_screen(self):
        self.must_update = True
        if self.current_item + self.max_rows <= self.selected:
            self.move_to(1)
            self.set_selected_on_screen()
        elif self.current_item > self.selected:
            self.move_to(-1)
            self.set_selected_on_screen()

    def reload_selected(self):
        self.must_update = True
        if self.selected is not None:
            try:
                self.screen_objects.get_touch_object(
                    str(self.selected)).set_selected(
                        True)
            except KeyError:
                pass



import logging
import time

import pygame

logger = logging.getLogger(__name__)


class InputManager():
    click = 1
    swipe = 2
    long_click = 3
    key = 4

    long_click_min_time = 0.3

    up = 0
    down = 1
    left = 2
    right = 3
    enter = 4

    special_keys = [pygame.K_DOWN, pygame.K_UP, pygame.K_LEFT,
                    pygame.K_RIGHT, pygame.K_RETURN]

    def __init__(self, size):
        self.down_pos = (0, 0)
        self.up_pos = (0, 0)
        self.screen_size = size
        self.max_move_margin = self.screen_size[1] / 6
        self.min_swipe_move = self.screen_size[1] / 3
        self.down_button = -1
        self.down_time = -1
        self.last_key = -1

    def event(self, event):

        if event.type == pygame.MOUSEBUTTONUP:
            if event.button == 4:
                touch_event = InputEvent(InputManager.swipe,
                                         self.down_pos,
                                         self.up_pos, True,
                                         InputManager.up, None)
                return touch_event
            elif event.button == 5:
                touch_event = InputEvent(InputManager.swipe,
                                         self.down_pos,
                                         self.up_pos, True,
                                         InputManager.down, None)
                return touch_event
            elif event.button == 1 and self.down_button == 1:
                touch_event = self.mouse_up(event)
                return touch_event
            elif event.button == 3 and self.down_button == 3:
                touch_event = InputEvent(InputManager.long_click,
                                         self.down_pos, self.up_pos,
                                         None, None, None)
                return touch_event
            else:
                return None
        elif event.type == pygame.MOUSEBUTTONDOWN:
            self.mouse_down(event)
            return None
        elif event.type == pygame.KEYDOWN:
            return self.key_down(event)
        elif event.type == pygame.KEYUP:
            return self.key_up(event)

    def key_down(self, event):
        if event.unicode is not None\
                and len(event.unicode) > 0 and event.key not in \
                InputManager.special_keys:
            return InputEvent(InputManager.key, None, None, None,
                              None, event.unicode)
        else:
            self.last_key = event.key
            self.down_time = time.time()
            return None

    def key_up(self, event):
        if self.last_key == event.key:
            if self.last_key == pygame.K_DOWN:
                direction = InputManager.down
            elif self.last_key == pygame.K_UP:
                direction = InputManager.up
            elif self.last_key == pygame.K_LEFT:
                direction = InputManager.left
            elif self.last_key == pygame.K_RIGHT:
                direction = InputManager.right
            elif self.last_key == pygame.K_RETURN:
                direction = InputManager.enter
            else:
                return None
            if direction is not None:
                if time.time() - self.down_time > \
                        InputManager.long_click_min_time:
                    longpress = True
                else:
                    longpress = False
                return InputEvent(InputManager.key, None, None, None,
                                  direction, self.last_key,
                                  longpress=longpress)

    def mouse_down(self, event):
        self.down_pos = event.pos
        self.down_button = event.button
        self.down_time = time.time()

    def mouse_up(self, event):
        self.up_pos = event.pos
        if abs(self.down_pos[0] -
                self.up_pos[0]) < self.max_move_margin:
            if abs(self.down_pos[1] -
                    self.up_pos[1]) < self.max_move_margin:
                if time.time() - InputManager.long_click_min_time > \
                        self.down_time:
                    return InputEvent(InputManager.long_click,
                                      self.down_pos,
                                      self.up_pos, None, None)
                else:
                    return InputEvent(InputManager.click,
                                      self.down_pos,
                                      self.up_pos, None, None)
            elif abs(self.down_pos[1] - self.up_pos[1])\
                    > self.min_swipe_move:
                return InputEvent(InputManager.swipe, self.down_pos,
                                  self.up_pos, True, None)
        elif self.down_pos[1] - self.up_pos[1] < self.max_move_margin:
            if abs(self.down_pos[0] - self.up_pos[0]) > \
                    self.min_swipe_move:
                return InputEvent(InputManager.swipe, self.down_pos,
                                  self.up_pos, False, None)


class InputEvent:
    def __init__(self, event_type, down_pos, current_pos, vertical,
                 direction, unicode=None, longpress=False):
        self.type = event_type
        self.down_pos = down_pos
        self.current_pos = current_pos
        self.unicode = unicode
        self.longpress = longpress
        if event_type is InputManager.swipe and direction is None:
            if vertical:
                if self.down_pos[1] < self.current_pos[1]:
                    self.direction = InputManager.down
                else:
                    self.direction = InputManager.up
            else:
                if self.down_pos[0] < self.current_pos[0]:
                    self.direction = InputManager.right
                else:
                    self.direction = InputManager.left
        else:
            self.direction = direction



import os
import socket

from base_screen import BaseScreen

from ..graphic_utils import ListView


class MenuScreen(BaseScreen):
    def __init__(self, size, base_size, manager, fonts, core):
        BaseScreen.__init__(self, size, base_size, manager, fonts)
        self.ip = None
        self.core = core
        self.list = ListView((0, 0), (size[0], size[1]-base_size),
                             base_size, fonts['base'])

        self.list_items = ["Random", "Repeat", "Single", "Consume",
                           "Exit Mopidy", "Shutdown", "Restart", "IP: "]

        self.list.set_list(self.list_items)

    def should_update(self):
        return self.list.should_update()

    def update(self, screen, update_type, rects):
        update_all = (update_type == BaseScreen.update_all)
        self.list.render(screen, update_all, rects)

    def touch_event(self, event):
        clicked = self.list.touch_event(event)
        if clicked is not None:
            if clicked == 0:
                random = not self.core.tracklist.get_random().get()
                self.core.tracklist.set_random(random)
            elif clicked == 1:
                repeat = not self.core.tracklist.get_repeat().get()
                self.core.tracklist.set_repeat(repeat)
            elif clicked == 2:
                single = not self.core.tracklist.get_single().get()
                self.core.tracklist.set_single(single)
            elif clicked == 3:
                consume = not self.core.tracklist.get_consume().get()
                self.core.tracklist.set_consume(consume)
            elif clicked == 4:
                os.system("pkill mopidy")
            elif clicked == 5:
                if os.system("gksu -- shutdown now -h") != 0:
                    os.system("sudo shutdown now -h")
            elif clicked == 6:
                if os.system("gksu -- shutdown -r now") != 0:
                    os.system("sudo shutdown -r now")
            elif clicked == 7:
                self.check_connection()

    # Will check internet connection
    def check_connection(self):
        try:
            s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
            s.connect(("8.8.8.8", 80))
            self.ip = s.getsockname()[0]
            s.close()
            self.list_items[7] = "IP: " + self.ip
            self.list.set_list(self.list_items)
        except socket.error:
            s.close()
            self.ip = None
            self.list_items[7] = "IP: No internet"
            self.list.set_list(self.list_items)

    def options_changed(self):
        active = []
        if self.core.tracklist.random.get():
            active.append(0)
        if self.core.tracklist.repeat.get():
            active.append(1)
        if self.core.tracklist.single.get():
            active.append(2)
        if self.core.tracklist.consume.get():
            active.append(3)
        self.list.set_active(active)

    def set_connection(self, connection, loading):
        internet = self.touch_text_manager.get_touch_object(
            "internet")
        if loading:
            internet.set_text(u"\ue627", None)
            internet.set_active(False)
        else:
            internet.set_text(u"\ue602", None)
            internet.set_active(connection)



import logging
import traceback

from graphic_utils import DynamicBackground, \
    ScreenObjectsManager, TouchAndTextItem
from input import InputManager

from pkg_resources import Requirement, resource_filename

import pygame

from screens import BaseScreen, Keyboard, LibraryScreen, MainScreen, MenuScreen,\
    PlaylistScreen, SearchScreen, Tracklist


logger = logging.getLogger(__name__)

search_index = 0
main_screen_index = 1
tracklist_index = 2
library_index = 3
playlist_index = 4
menu_index = 5


class ScreenManager():

    def __init__(self, size, core, cache, resolution_factor):
        self.core = core
        self.cache = cache
        self.fonts = {}
        self.background = None
        self.current_screen = library_index

        # Init variables in init
        self.base_size = None
        self.size = None
        self.screens = None
        self.track = None
        self.input_manager = InputManager(size)
        self.down_bar_objects = ScreenObjectsManager()
        self.down_bar = None
        self.keyboard = None
        self.update_type = BaseScreen.update_all

        self.resolution_factor = resolution_factor

        self.init_manager(size)

    def init_manager(self, size):
        self.size = size

        self.background = DynamicBackground(self.size)
        self.base_size = self.size[1] / self.resolution_factor
        font = resource_filename(
            Requirement.parse("mopidy-touchscreen"),
            "mopidy_touchscreen/icomoon.ttf")
        self.fonts['base'] = pygame.font.SysFont("arial",
                                                 int(self.base_size*0.9))
        self.fonts['icon'] = pygame.font.Font(font, int(self.base_size*0.9))
        try:
            self.screens = [
                SearchScreen(size, self.base_size, self, self.fonts),
                MainScreen(size, self.base_size, self, self.fonts,
                           self.cache, self.core, self.background),
                Tracklist(size, self.base_size, self, self.fonts),
                LibraryScreen(size, self.base_size, self, self.fonts),
                PlaylistScreen(size,
                               self.base_size, self, self.fonts),
                MenuScreen(size, self.base_size, self, self.fonts, self.core)]
        except:
            traceback.print_exc()
        self.track = None

        # Menu buttons

        button_size = (self.size[0] / 6, self.base_size)

        # Search button
        button = TouchAndTextItem(self.fonts['icon'], u" \ue986",
                                  (0, self.size[1] - self.base_size),
                                  button_size, center=True)
        self.down_bar_objects.set_touch_object("menu_0", button)
        x = button.get_right_pos()

        # Main button
        button = TouchAndTextItem(self.fonts['icon'], u" \ue600",
                                  (x, self.size[1] - self.base_size),
                                  button_size, center=True)
        self.down_bar_objects.set_touch_object("menu_1", button)
        x = button.get_right_pos()

        # Tracklist button
        button = TouchAndTextItem(self.fonts['icon'], u" \ue60d",
                                  (x, self.size[1] - self.base_size),
                                  button_size, center=True)
        self.down_bar_objects.set_touch_object("menu_2", button)
        x = button.get_right_pos()

        # Library button
        button = TouchAndTextItem(self.fonts['icon'], u" \ue604",
                                  (x, self.size[1] - self.base_size),
                                  button_size, center=True)
        self.down_bar_objects.set_touch_object("menu_3", button)
        x = button.get_right_pos()

        # Playlist button
        button = TouchAndTextItem(self.fonts['icon'], u" \ue605",
                                  (x, self.size[1] - self.base_size),
                                  button_size, center=True)

        self.down_bar_objects.set_touch_object("menu_4", button)
        x = button.get_right_pos()

        # Menu button
        button = TouchAndTextItem(self.fonts['icon'], u" \ue60a",
                                  (x, self.size[1] - self.base_size),
                                  button_size,
                                  center=True)
        self.down_bar_objects.set_touch_object("menu_5", button)

        # Down bar
        self.down_bar = pygame.Surface(
            (self.size[0], self.size[1] - self.base_size),
            pygame.SRCALPHA)
        self.down_bar.fill((0, 0, 0, 200))

        self.options_changed()
        self.mute_changed(self.core.playback.mute.get())
        playback_state = self.core.playback.state.get()
        self.playback_state_changed(playback_state,
                                    playback_state)
        self.screens[menu_index].check_connection()

        self.change_screen(self.current_screen)

        self.update_type = BaseScreen.update_all

    def get_update_type(self):
        if self.update_type == BaseScreen.update_all:
            self.update_type = BaseScreen.no_update
            return BaseScreen.update_all
        else:
            if self.keyboard:
                return BaseScreen.no_update
            else:
                if self.background.should_update():
                    return BaseScreen.update_all
                else:
                    if self.screens[self.current_screen].should_update():
                        return BaseScreen.update_partial
                    else:
                        return BaseScreen.no_update

    def update(self, screen):
        update_type = self.get_update_type()
        if update_type != BaseScreen.no_update:
            rects = []
            surface = self.background.draw_background()
            if self.keyboard:
                self.keyboard.update(surface)
            else:
                self.screens[self.current_screen].\
                    update(surface, update_type, rects)
                surface.blit(self.down_bar, (0, self.size[1] - self.base_size))
                self.down_bar_objects.render(surface)

            if update_type == BaseScreen.update_all or len(rects) < 1:
                screen.blit(surface, (0, 0))
                pygame.display.flip()
            else:
                for rect in rects:
                    screen.blit(surface, rect, area=rect)
                pygame.display.update(rects)

    def track_started(self, track):
        self.track = track
        self.screens[main_screen_index].track_started(track.track)
        self.screens[tracklist_index].track_started(track)

    def track_playback_ended(self, tl_track, time_position):
        self.screens[main_screen_index].track_playback_ended(
            tl_track, time_position)

    def event(self, event):
        event = self.input_manager.event(event)
        if event is not None:
            if self.keyboard is not None:
                self.keyboard.touch_event(event)
            elif not self.manage_event(event):
                self.screens[self.current_screen].touch_event(event)
            self.update_type = BaseScreen.update_all

    def manage_event(self, event):
        if event.type == InputManager.click:
            objects = \
                self.down_bar_objects.get_touch_objects_in_pos(
                    event.current_pos)
            return self.click_on_objects(objects, event)
        else:
            if event.type == InputManager.key and not event.longpress:
                dir = event.direction
                if dir == InputManager.right or dir == InputManager.left:
                    if not self.screens[self.current_screen]\
                            .change_screen(dir):
                        if dir == InputManager.right:
                            self.change_screen(self.current_screen+1)
                        else:
                            self.change_screen(self.current_screen-1)
                    return True
            return False

    def volume_changed(self, volume):
        self.screens[main_screen_index].volume_changed(volume)
        self.update_type = BaseScreen.update_all

    def playback_state_changed(self, old_state, new_state):
        self.screens[main_screen_index].playback_state_changed(
            old_state, new_state)
        self.update_type = BaseScreen.update_all

    def mute_changed(self, mute):
        self.screens[main_screen_index].mute_changed(mute)
        self.update_type = BaseScreen.update_all

    def tracklist_changed(self):
        self.screens[tracklist_index].tracklist_changed()
        self.update_type = BaseScreen.update_all

    def options_changed(self):
        self.screens[menu_index].options_changed()
        self.update_type = BaseScreen.update_all

    def change_screen(self, new_screen):
        if new_screen > -1 and new_screen < len(self.screens):
            self.down_bar_objects.get_touch_object(
                "menu_" + str(self.current_screen)).set_active(False)
            self.current_screen = new_screen
            self.down_bar_objects.get_touch_object(
                "menu_" + str(new_screen)).set_active(True)
        self.update_type = BaseScreen.update_all

    def click_on_objects(self, objects, event):
        if objects is not None:
            for key in objects:
                if key[:-1] == "menu_":
                    self.change_screen(int(key[-1:]))
                    return True
        return False

    def playlists_loaded(self):
        self.screens[playlist_index].playlists_loaded()
        self.update_type = BaseScreen.update_all

    def search(self, query, mode):
        self.screens[search_index].search(query, mode)
        self.update_type = BaseScreen.update_all

    def resize(self, event):
        self.init_manager(event.size)
        self.update_type = BaseScreen.update_all

    def open_keyboard(self, input_listener):
        self.keyboard = Keyboard(self.size, self.base_size, self,
                                 self.fonts, input_listener)
        self.update_type = BaseScreen.update_all

    def close_keyboard(self):
        self.keyboard = None
        self.update_type = BaseScreen.update_all



import hashlib
import json
import logging
import os
import time
import urllib
import urllib2
from threading import Thread


from base_screen import BaseScreen

import mopidy.core

import pygame

from ..graphic_utils import Progressbar, \
    ScreenObjectsManager, TextItem, TouchAndTextItem
from ..input import InputManager


logger = logging.getLogger(__name__)


class MainScreen(BaseScreen):
    def __init__(self, size, base_size, manager, fonts, cache, core,
                 background):
        BaseScreen.__init__(self, size, base_size, manager, fonts)
        self.core = core
        self.track = None
        self.cache = cache
        self.image = None
        self.artists = None
        self.update_next_frame = True
        self.background = background
        self.update_keys = []
        self.current_track_pos = 0
        self.track_duration = "00:00"
        self.touch_text_manager = ScreenObjectsManager()
        current_track = self.core.playback.current_track.get()
        if current_track is None:
            self.track_playback_ended(None, None)
        else:
            self.track_started(current_track)

        # Top bar
        self.top_bar = pygame.Surface((self.size[0], self.base_size),
                                      pygame.SRCALPHA)
        self.top_bar.fill((0, 0, 0, 128))

        # Play/pause
        button = TouchAndTextItem(self.fonts['icon'], u"\ue615 ",
                                  (0, 0), None)
        self.touch_text_manager.set_touch_object("pause_play", button)
        x = button.get_right_pos()

        # Mute
        button = TouchAndTextItem(self.fonts['icon'], u"\ue61f ",
                                  (x, 0), None)
        self.touch_text_manager.set_touch_object("mute", button)
        x = button.get_right_pos()

        # Volume
        progress = Progressbar(self.fonts['base'], "100", (x, 0),
                               (self.size[0] - x, self.base_size),
                               100, True)
        self.touch_text_manager.set_touch_object("volume", progress)
        progress.set_value(self.core.playback.volume.get())

    def should_update(self):
        if len(self.update_keys) > 0:
            return True
        else:
            track_pos_millis = self.core.playback.time_position.get()
            new_track_pos = track_pos_millis / 1000
            if new_track_pos != self.current_track_pos:
                return True
            else:
                return False

    def update(self, screen, update_type, rects):
        if update_type == BaseScreen.update_all:
            screen.blit(self.top_bar, (0, 0))
            self.touch_text_manager.render(screen)
            if self.image is not None:
                screen.blit(self.image, (
                    self.base_size / 2, self.base_size +
                    self.base_size / 2))

        elif update_type == BaseScreen.update_partial \
                and self.track is not None:
            track_pos_millis = self.core.playback.time_position.get()
            new_track_pos = track_pos_millis / 1000

            if new_track_pos != self.current_track_pos:
                progress = self.touch_text_manager.get_touch_object(
                    "time_progress")
                progress.set_value(track_pos_millis)
                self.current_track_pos = new_track_pos
                progress.set_text(
                    time.strftime('%M:%S', time.gmtime(
                        self.current_track_pos)) +
                    "/" + self.track_duration)
                progress.render(screen)
                rects.append(progress.rect_in_pos)

            for key in self.update_keys:
                object = self.touch_text_manager.get_object(key)
                object.update()
                object.render(screen)
                rects.append(object.rect_in_pos)

    def track_started(self, track):
        self.update_keys = []
        self.image = None
        x = self.size[1] - self.base_size * 3
        width = self.size[0] - self.base_size / 2 - x

        self.track_duration = time.strftime('%M:%S', time.gmtime(
            track.length / 1000))

        # Load all artists
        self.artists = []
        for artist in track.artists:
            self.artists.append(artist)

        # Track name
        label = TextItem(self.fonts['base'],
                         MainScreen.get_track_name(track),
                         (x, (self.size[1]-self.base_size*3)/2
                          - self.base_size*0.5),
                         (width, -1))
        if not label.fit_horizontal:
            self.update_keys.append("track_name")
        self.touch_text_manager.set_object("track_name", label)

        # Album name
        label = TextItem(self.fonts['base'],
                         MainScreen.get_track_album_name
                         (track),
                         (x, (self.size[1]-self.base_size*3)/2
                          + self.base_size*0.5),
                         (width, -1))
        if not label.fit_horizontal:
            self.update_keys.append("album_name")
        self.touch_text_manager.set_object("album_name", label)

        # Artist
        label = TextItem(self.fonts['base'],
                         self.get_artist_string(),
                         (x, (self.size[1]-self.base_size*3)/2
                          + self.base_size*1.5),
                         (width, -1))
        if not label.fit_horizontal:
            self.update_keys.append("artist_name")
        self.touch_text_manager.set_object("artist_name", label)

        # Previous track button
        button = TouchAndTextItem(self.fonts['icon'], u"\ue61c",
                                  (0, self.size[1] - self.base_size * 2), None)
        self.touch_text_manager.set_touch_object("previous", button)
        size_1 = button.get_right_pos()

        size_2 = self.fonts['icon'].size(u"\ue61d")[0]
        button = TouchAndTextItem(self.fonts['icon'], u"\ue61d",
                                  (self.size[0] - size_2,
                                   self.size[1] - self.base_size * 2),
                                  None)
        self.touch_text_manager.set_touch_object("next", button)

        # Progress
        progress = Progressbar(self.fonts['base'],
                               time.strftime('%M:%S', time.gmtime(
                                   0)) + "/" + time.strftime(
                                   '%M:%S', time.gmtime(0)),
                               (size_1, self.size[1] - self.base_size * 2),
                               (
                                   self.size[0] - size_1 - size_2,
                                   self.base_size),
                               track.length, False)
        self.touch_text_manager.set_touch_object("time_progress",
                                                 progress)

        self.track = track
        if not self.is_image_in_cache():
            thread = Thread(target=self.download_image)
            thread.start()
        else:
            thread = Thread(target=self.load_image)
            thread.start()

    def get_artist_string(self):
        artists_string = ''
        for artist in self.artists:
            artists_string += artist.name + ', '
        if len(artists_string) > 2:
            artists_string = artists_string[:-2]
        elif len(artists_string) == 0:
            artists_string = "Unknow Artist"
        return artists_string

    def get_image_file_name(self):
        name = MainScreen.get_track_album_name(
            self.track) + '-' + self.get_artist_string()
        md5name = hashlib.md5(name.encode('utf-8')).hexdigest()
        return md5name

    def get_cover_folder(self):
        if not os.path.isdir(self.cache + "/covers"):
            os.makedirs(self.cache + "/covers")
        return self.cache + "/covers/"

    def is_image_in_cache(self):
        self.get_cover_folder()
        return os.path.isfile(
            self.get_cover_folder() + self.get_image_file_name())

    def download_image(self):
        image_uris = self.core.library.get_images(
            {self.track.uri}).get()[self.track.uri]
        if len(image_uris) > 0:
            urllib.urlretrieve(image_uris[0].uri,
                               self.get_cover_folder() +
                               self.get_image_file_name())
            self.load_image()
        else:
            self.download_image_last_fm(0)

    def download_image_last_fm(self, artist_index):
        if artist_index < len(self.artists):
            try:
                safe_artist = urllib.quote_plus(
                    self.artists[artist_index].name)
                safe_album = urllib.quote_plus(
                    MainScreen.get_track_album_name(self.track))
                url = "http://ws.audioscrobbler.com/2.0/?"
                params = "method=album.getinfo&" + \
                         "api_key=59a04c6a73fb99d6e8996e01db306829&" \
                         + "artist=" \
                         + safe_artist + "&album=" + safe_album + \
                         "&format=json"
                response = urllib2.urlopen(url + params)
                data = json.load(response)
                image = data['album']['image'][-1]['#text']
                urllib.urlretrieve(image,
                                   self.get_cover_folder() +
                                   self.get_image_file_name())
                self.load_image()
            except:
                self.download_image_last_fm(artist_index + 1)
        else:

            logger.info("Cover could not be downloaded")

            # There is no cover
            # so it will use all the screen size for the text
            width = self.size[0] - self.base_size

            current = TextItem(self.fonts['base'],
                               MainScreen.get_track_name
                               (self.track),
                               (self.base_size / 2,
                                self.base_size * 2),
                               (width, -1))
            if not current.fit_horizontal:
                self.update_keys.append("track_name")
            self.touch_text_manager.set_object("track_name", current)

            current = TextItem(self.fonts['base'],
                               MainScreen.get_track_album_name
                               (self.track),
                               (self.base_size / 2,
                                self.base_size * 3),
                               (width, -1))
            if not current.fit_horizontal:
                self.update_keys.append("album_name")
            self.touch_text_manager.set_object("album_name", current)

            current = TextItem(self.fonts['base'],
                               self.get_artist_string(),
                               (self.base_size / 2,
                                self.base_size * 4),
                               (width, -1))
            if not current.fit_horizontal:
                self.update_keys.append("artist_name")
            self.touch_text_manager.set_object("artist_name", current)

            self.background.set_background_image(None)

    def track_playback_ended(self, tl_track, time_position):
        self.background.set_background_image(None)
        self.image = None
        self.track_duration = "00:00"

        width = self.size[0] - self.base_size

        current = TextItem(self.fonts['base'], "",
                           (self.base_size / 2,
                            self.base_size * 2),
                           (width, -1))
        self.touch_text_manager.set_object("track_name", current)

        current = TextItem(self.fonts['base'], "",
                           (self.base_size / 2,
                            self.base_size * 3),
                           (width, -1))
        self.touch_text_manager.set_object("album_name", current)

        current = TextItem(self.fonts['base'], "",
                           (self.base_size / 2,
                            self.base_size * 4),
                           (width, -1))
        self.touch_text_manager.set_object("artist_name", current)

    def load_image(self):
        size = self.size[1] - self.base_size * 4
        image_original = pygame.image.load(
            self.get_cover_folder() +
            self.get_image_file_name())
        image = pygame.transform.scale(image_original, (size, size))
        image = image.convert()
        self.image = image
        self.background.set_background_image(image_original)

    def touch_event(self, event):
        if event.type == InputManager.click:
            objects = \
                self.touch_text_manager.get_touch_objects_in_pos(
                    event.current_pos)
            if objects is not None:
                self.click_on_objects(objects, event)

        elif event.type == InputManager.swipe:
            if event.direction == InputManager.left:
                self.core.playback.next()
            elif event.direction == InputManager.right:
                self.core.playback.previous()
            elif event.direction == InputManager.up:
                volume = self.core.playback.volume.get() + 10
                if volume > 100:
                    volume = 100
                self.core.mixer.set_volume(volume)
            elif event.direction == InputManager.down:
                volume = self.core.playback.volume.get() - 10
                if volume < 0:
                    volume = 0
                self.core.mixer.set_volume(volume)
        elif event.type == InputManager.key:
            if event.direction == InputManager.enter:
                self.click_on_objects(["pause_play"], None)
            elif event.direction == InputManager.up:
                vol = self.core.mixer.get_volume().get()
                vol += 3
                if vol > 100:
                    vol = 100
                self.core.mixer.set_volume(vol)
            elif event.direction == InputManager.down:
                vol = self.core.mixer.get_volume().get()
                vol -= 3
                if vol < 0:
                    vol = 0
                self.core.mixer.set_volume(vol)
            elif event.longpress:
                if event.direction == InputManager.left:
                    self.click_on_objects(["previous"], None)
                elif event.direction == InputManager.right:
                    self.click_on_objects(["next"], None)

    def click_on_objects(self, objects, event):
        if objects is not None:
            for key in objects:
                if key == "time_progress":
                    value = self.touch_text_manager.get_touch_object(
                        key).get_pos_value(
                        event.current_pos)
                    self.core.playback.seek(value)

                elif key == "previous":
                    self.core.playback.previous()
                elif key == "next":
                    self.core.playback.next()
                elif key == "volume":
                    self.change_volume(event)
                elif key == "pause_play":
                    if self.core.playback.state.get() == \
                            mopidy.core.PlaybackState.PLAYING:
                        self.core.playback.pause()
                    else:
                        self.core.playback.play()
                elif key == "mute":
                    mute = not self.core.playback.mute.get()
                    self.core.playback.set_mute(mute)
                    self.mute_changed(mute)

    def change_volume(self, event):
        manager = self.touch_text_manager
        volume = manager.get_touch_object("volume")
        pos = event.current_pos
        value = volume.get_pos_value(pos)
        self.core.mixer.set_volume(value)

    def playback_state_changed(self, old_state, new_state):
        if new_state == mopidy.core.PlaybackState.PLAYING:
            self.touch_text_manager.get_touch_object(
                "pause_play").set_text(u"\ue616", False)
        else:
            self.touch_text_manager.get_touch_object(
                "pause_play").set_text(u"\ue615", False)

    def volume_changed(self, volume):
        if not self.core.playback.mute.get():
            if volume > 80:
                self.touch_text_manager.get_touch_object(
                    "mute").set_text(
                    u"\ue61f", False)
            elif volume > 50:
                self.touch_text_manager.get_touch_object(
                    "mute").set_text(
                    u"\ue620", False)
            elif volume > 20:
                self.touch_text_manager.get_touch_object(
                    "mute").set_text(
                    u"\ue621", False)
            else:
                self.touch_text_manager.get_touch_object(
                    "mute").set_text(
                    u"\ue622", False)
        self.touch_text_manager.get_touch_object("volume").set_value(
            volume)

    def mute_changed(self, mute):
        self.touch_text_manager.get_touch_object("mute").set_active(
            not mute)
        if mute:
            self.touch_text_manager.get_touch_object("mute").set_text(
                u"\ue623", False)
        else:
            self.volume_changed(self.core.playback.volume.get())

    @staticmethod
    def get_track_name(track):
        if track.name is None:
            return track.uri
        else:
            return track.name

    @staticmethod
    def get_track_album_name(track):
        if track.album is not None and track.album.name is not None \
                and len(track.album.name) > 0:
            return track.album.name
        else:
            return "Unknow Album"



from base_screen import BaseScreen

import pygame

from ..graphic_utils import ListView,\
    ScreenObjectsManager, TouchAndTextItem

from ..input import InputManager

mode_track_name = 0
mode_album_name = 1
mode_artist_name = 2


class SearchScreen(BaseScreen):
    def __init__(self, size, base_size, manager, fonts):
        BaseScreen.__init__(self, size, base_size, manager, fonts)
        self.list_view = ListView((0, self.base_size*2), (
            self.size[0], self.size[1] -
            3*self.base_size), self.base_size, manager.fonts['base'])
        self.results_strings = []
        self.results = []
        self.screen_objects = ScreenObjectsManager()
        self.query = ""

        # Search button
        button = TouchAndTextItem(self.fonts['icon'], u" \ue986",
                                  (0, self.base_size),
                                  None, center=True)
        self.screen_objects.set_touch_object(
            "search", button)

        x = button.get_right_pos()

        # Query text
        text = TouchAndTextItem(self.fonts['base'], self.query, (0, 0),
                                (self.size[0], self.base_size), center=True)
        self.screen_objects.set_touch_object("query", text)

        # Mode buttons
        button_size = ((self.size[0]-x)/3, self.base_size)
        self.mode_objects_keys = ["mode_track", "mode_album",
                                  "mode_artist"]

        # Track button
        button = TouchAndTextItem(self.fonts['base'], "Track",
                                  (x, self.base_size),
                                  (button_size[0], self.base_size),
                                  center=True)
        self.screen_objects.set_touch_object(
            self.mode_objects_keys[0], button)

        # Album button
        button = TouchAndTextItem(self.fonts['base'], "Album",
                                  (button_size[0]+x, self.base_size),
                                  button_size, center=True)
        self.screen_objects.set_touch_object(
            self.mode_objects_keys[1], button)

        # Artist button
        button = TouchAndTextItem(self.fonts['base'], "Artist",
                                  (button_size[0]*2+x, self.base_size),
                                  button_size, center=True)
        self.screen_objects.set_touch_object(
            self.mode_objects_keys[2], button)

        # Top Bar
        self.top_bar = pygame.Surface(
            (self.size[0], self.base_size * 2),
            pygame.SRCALPHA)
        self.top_bar.fill((0, 0, 0, 128))
        self.mode = -1
        self.set_mode(mode=mode_track_name)
        self.set_query("Search")

    def should_update(self):
        return self.list_view.should_update()

    def update(self, screen, update_type, rects):
        screen.blit(self.top_bar, (0, 0))
        self.screen_objects.render(screen)
        update_all = (update_type == BaseScreen.update_all)
        self.list_view.render(screen, update_all, rects)

    def set_mode(self, mode=mode_track_name):
        if mode is not self.mode:
            self.mode = mode
            for key in self.mode_objects_keys:
                self.screen_objects.get_touch_object(key).\
                    set_active(False)
            self.screen_objects.get_touch_object(
                self.mode_objects_keys[self.mode]).set_active(True)

    def set_query(self, query=""):
        self.query = query
        self.screen_objects.get_touch_object("query").set_text(
            self.query, False)

    def search(self, query=None, mode=None):
        if query is not None:
            self.set_query(query)
        if mode is not None:
            self.set_mode(mode)
        if self.mode == mode_track_name:
            search_query = {'any': [self.query]}
        elif self.mode == mode_album_name:
            search_query = {'album': [self.query]}
        else:
            search_query = {'artist': [self.query]}
        if len(self.query) > 0:
            current_results = self.manager.core.library.search(
                search_query).get()
            self.results = []
            self.results_strings = []
            for backend in current_results:
                if mode == mode_track_name:
                    iterable = backend.tracks
                elif mode == mode_album_name:
                    iterable = backend.albums
                else:
                    iterable = backend.artists

                for result in iterable:
                    self.results.append(result)
                    self.results_strings.append(result.name)
            self.list_view.set_list(self.results_strings)

    def touch_event(self, touch_event):
        if touch_event.type == InputManager.click:
            clicked = self.list_view.touch_event(touch_event)
            if clicked is not None:
                self.manager.core.tracklist.clear()
                self.manager.core.tracklist.add(
                    uri=self.results[clicked].uri)
                self.manager.core.playback.play()
            else:
                clicked = self.screen_objects.get_touch_objects_in_pos(
                    touch_event.down_pos)
                if len(clicked) > 0:
                    clicked = clicked[0]
                    if clicked == self.mode_objects_keys[0]:
                        self.search(mode=0)
                    if clicked == self.mode_objects_keys[1]:
                        self.search(mode=1)
                    if clicked == self.mode_objects_keys[2]:
                        self.search(mode=2)
                    if clicked == "query" or clicked == "search":
                        self.manager.open_keyboard(self)
        else:
            pos = self.list_view.touch_event(touch_event)
            if pos is not None:
                self.manager.core.tracklist.clear()
                self.manager.core.tracklist.add(
                    uri=self.results[pos].uri)
                self.manager.core.playback.play()

    def change_screen(self, direction):
        if direction == InputManager.right:
            if self.mode < 2:
                self.set_mode(self.mode+1)
                return True
        elif direction == InputManager.left:
            if self.mode > 0:
                self.set_mode(self.mode-1)
                return True
            else:
                self.manager.open_keyboard(self)
        return False

    def text_input(self, text):
        self.search(text, self.mode)



# flake8: noqa
from base_screen import BaseScreen
from library_screen import LibraryScreen
from keyboard_screen import Keyboard
from main_screen import MainScreen
from menu_screen import MenuScreen
from playlist_screen import PlaylistScreen
from search_screen import SearchScreen
from tracklist import Tracklist


# from __future__ import unicode_literals

import unittest

from mopidy_touchscreen import Extension


# ,touch_screen as frontend_lib


class ExtensionTest(unittest.TestCase):

    def test_get_default_config(self):
        ext = Extension()

        config = ext.get_default_config()

        self.assertIn('[touchscreen]', config)
        self.assertIn('enabled = true', config)

    def test_get_config_schema(self):
        pass
        # ext = Extension()

        # schema = ext.get_config_schema()

        # TODO Test the content of your config schema
        # self.assertIn('username', schema)
        # self.assertIn('password', schema)



from base_screen import BaseScreen

from .main_screen import MainScreen
from ..graphic_utils import ListView


class Tracklist(BaseScreen):
    def __init__(self, size, base_size, manager, fonts):
        BaseScreen.__init__(self, size, base_size, manager, fonts)
        self.size = size
        self.base_size = base_size
        self.manager = manager
        self.list_view = ListView((0, 0), (
            self.size[0], self.size[1] -
            self.base_size), self.base_size, self.fonts['base'])
        self.tracks = []
        self.tracks_strings = []
        self.update_list()
        self.track_started(
            self.manager.core.playback.current_tl_track.get())

    def should_update(self):
        return self.list_view.should_update()

    def update(self, screen, update_type, rects):
        update_all = (update_type == BaseScreen.update_all)
        self.list_view.render(screen, update_all, rects)

    def tracklist_changed(self):
        self.update_list()

    def update_list(self):
        self.tracks = self.manager.core.tracklist.tl_tracks.get()
        self.tracks_strings = []
        for tl_track in self.tracks:
            self.tracks_strings.append(
                MainScreen.get_track_name(tl_track.track))
        self.list_view.set_list(self.tracks_strings)

    def touch_event(self, touch_event):
        pos = self.list_view.touch_event(touch_event)
        if pos is not None:
            self.manager.core.playback.play(self.tracks[pos])

    def track_started(self, track):
        self.list_view.set_active(
            [self.manager.core.tracklist.index(track).get()])



from base_screen import BaseScreen

from ..graphic_utils import ListView


class PlaylistScreen(BaseScreen):
    def __init__(self, size, base_size, manager, fonts):
        BaseScreen.__init__(self, size, base_size, manager, fonts)
        self.list_view = ListView((0, 0), (
            self.size[0], self.size[1] -
            self.base_size), self.base_size,
            self.fonts['base'])
        self.playlists_strings = []
        self.playlists = []
        self.selected_playlist = None
        self.playlist_tracks = []
        self.playlist_tracks_strings = []
        self.playlists_loaded()

    def should_update(self):
        return self.list_view.should_update()

    def update(self, screen, update_type, rects):
        update_all = (update_type == BaseScreen.update_all)
        self.list_view.render(screen, update_all, rects)

    def playlists_loaded(self):
        self.selected_playlist = None
        self.playlists_strings = []
        self.playlists = []
        for playlist in self.manager.core.playlists.playlists.get():
            self.playlists.append(playlist)
            self.playlists_strings.append(playlist.name)
        self.list_view.set_list(self.playlists_strings)

    def playlist_selected(self, playlist):
        self.selected_playlist = playlist
        self.playlist_tracks = playlist.tracks
        self.playlist_tracks_strings = ["../"]
        for track in self.playlist_tracks:
            self.playlist_tracks_strings.append(track.name)
        self.list_view.set_list(self.playlist_tracks_strings)

    def touch_event(self, touch_event):
        clicked = self.list_view.touch_event(touch_event)
        if clicked is not None:
            if self.selected_playlist is None:
                self.playlist_selected(self.playlists[clicked])
            else:
                if clicked == 0:
                    self.selected_playlist = None
                    self.list_view.set_list(self.playlists_strings)
                else:
                    self.manager.core.tracklist.clear()
                    self.manager.core.tracklist.add(
                        self.playlist_tracks)
                    self.manager.core.playback.play(
                        tl_track=self.manager.core.
                        tracklist.tl_tracks.get()
                        [clicked-1])



import logging
import math

import pygame


logger = logging.getLogger(__name__)


class ScreenObjectsManager:
    def __init__(self):
        self.touch_objects = {}
        self.text_objects = {}
        self.selected = None
        self.selected_key = None

    def clear(self):
        self.touch_objects = {}
        self.text_objects = {}

    def set_object(self, key, add_object):
        self.text_objects[key] = add_object

    def get_object(self, key):
        return self.text_objects[key]

    def set_touch_object(self, key, add_object):
        self.touch_objects[key] = add_object

    def get_touch_object(self, key):
        return self.touch_objects[key]

    def render(self, surface):
        for key in self.text_objects:
            self.text_objects[key].update()
            self.text_objects[key].render(surface)
        for key in self.touch_objects:
            self.touch_objects[key].update()
            self.touch_objects[key].render(surface)

    def get_touch_objects_in_pos(self, pos):
        touched_objects = []
        for key in self.touch_objects:
            if self.touch_objects[key].is_pos_inside(pos):
                touched_objects.append(key)
        return touched_objects

    def clear_touch(self, not_remove):
        if not_remove is not None:
            new_touch = {}
            for key in not_remove:
                new_touch[key] = self.get_touch_object(key)
            self.touch_objects = new_touch
        else:
            self.touch_objects = {}

    def set_selected(self, key):
        if self.selected is not None:
            self.selected.set_selected(False)
        if key is not None:
            self.selected = self.touch_objects[key]
            self.selected.set_selected(True)
            self.selected_key = key
        else:
            self.selected = None
            self.selected_key = None


class BaseItem():
    def __init__(self, pos, size):
        self.pos = pos
        self.size = size
        self.rect = pygame.Rect(0, 0, self.size[0], self.size[1])
        self.rect_in_pos = pygame.Rect(self.pos[0], self.pos[1],
                                       self.size[0],
                                       self.size[1])

    def get_right_pos(self):
        return self.pos[0] + self.size[0]

    def update(self):
        return False


class TextItem(BaseItem):

    scroll_speed = 2

    def __init__(self, font, text, pos, size, center=False, background=None,
                 scroll_no_fit=True):
        self.font = font
        self.text = text
        self.scroll_no_fit = scroll_no_fit
        self.color = (255, 255, 255)
        self.box = self.font.render(text, True, self.color)
        self.box = self.box.convert_alpha()
        self.background = background
        if size is not None:
            if size[1] == -1:
                height = self.font.size(text)[1]
                BaseItem.__init__(self, pos, (size[0], height))
            else:
                BaseItem.__init__(self, pos, size)
        else:
            BaseItem.__init__(self, pos, self.font.size(text))
        if size is not None:
            if self.pos[0] + self.box.get_rect().width > pos[0] + \
                    size[0]:
                self.fit_horizontal = False
                self.step = 0
                self.step_2 = None
                self.scroll_white_gap = self.font.get_height() * 4
            else:
                self.fit_horizontal = True
            if self.pos[1] + self.box.get_rect().height > pos[1] + \
                    size[1]:
                self.fit_vertical = False
            else:
                self.fit_vertical = True
        else:
            self.fit_horizontal = True
            self.fit_vertical = True
        self.margin = 0
        self.center = center
        if self.center:
            if self.fit_horizontal:
                self.margin = (self.size[0] -
                               self.box.get_rect().width)/2

    def update(self):
        if self.scroll_no_fit and not self.fit_horizontal:
            self.step += TextItem.scroll_speed
            if self.step_2 is None:
                if (self.box.get_rect().width - self.step +
                        self.scroll_white_gap) < self.size[0]:
                    self.step_2 = \
                        self.box.get_rect().width - \
                        self.step + self.scroll_white_gap
            else:
                self.step_2 -= TextItem.scroll_speed
                if self.step_2 < 0:
                    self.step = 0 - self.step_2
                    self.step_2 = None
            return True
        else:
            return BaseItem.update(self)

    def render(self, surface):
        if self.background:
            surface.fill(self.background, rect=self.rect_in_pos)
            pygame.draw.rect(surface, (0, 0, 0), self.rect_in_pos, 1)
        if self.fit_horizontal:
            surface.blit(
                self.box, ((self.pos[0] + self.margin),
                           self.pos[1]), area=self.rect)
        else:
            if self.scroll_no_fit:
                surface.blit(self.box, self.pos,
                             area=pygame.Rect(self.step, 0, self.size[0],
                                              self.size[1]))
                if self.step_2 is not None:
                    surface.blit(self.box, (self.pos[0]+self.step_2,
                                            self.pos[1]),
                                 area=pygame.Rect(0, 0,
                                                  self.size[0] -
                                                  self.step_2,
                                                  self.size[1]))
            else:
                step = self.box.get_rect().width - self.size[0]
                surface.blit(self.box, self.pos,
                             area=pygame.Rect(step, 0, self.size[0],
                                              self.size[1]))

    def set_text(self, text, change_size):
        if text != self.text:
            if change_size:
                TextItem.__init__(self, self.font, text, self.pos,
                                  None, self.center, self.background,
                                  self.scroll_no_fit)
            else:
                TextItem.__init__(self, self.font, text, self.pos,
                                  self.size, self.center, self.background,
                                  self.scroll_no_fit)

    def add_text(self, add_text, change_size):
        self.set_text(self.text+add_text, change_size)

    def remove_text(self, chars, change_size):
        self.set_text(self.text[:-chars], change_size)


class TouchObject(BaseItem):
    def __init__(self, pos, size):
        BaseItem.__init__(self, pos, size)
        self.active = False
        self.selected = False
        self.selected_box = pygame.Surface(size, pygame.SRCALPHA)
        self.selected_box = self.selected_box.convert_alpha()
        self.selected_box.fill((0, 0, 0, 128))
        self.selected_box_rectangle = pygame.Surface(size, pygame.SRCALPHA)
        self.selected_box_rectangle = \
            self.selected_box_rectangle.convert_alpha()
        pygame.draw.rect(self.selected_box_rectangle, (255, 255, 255),
                         self.selected_box_rectangle.get_rect(),
                         size[1]/10+1)

    def is_pos_inside(self, pos):
        return self.rect_in_pos.collidepoint(pos)

    def set_active(self, active):
        self.active = active

    def set_selected(self, selected):
        self.selected = selected

    def render(self, surface):
        if self.selected:
            surface.blit(self.selected_box, self.pos)
            surface.blit(self.selected_box_rectangle, self.pos)

    def pre_render(self, surface):
        if self.selected:
            surface.blit(self.selected_box, self.pos)

    def post_render(self, surface):
        if self.selected:
            surface.blit(self.selected_box_rectangle, self.pos)


class TouchAndTextItem(TouchObject, TextItem):
    def __init__(self, font, text, pos, size, center=False, background=None,
                 scroll_no_fit=True):
        TextItem.__init__(self, font, text, pos, size, center=center,
                          background=background, scroll_no_fit=scroll_no_fit)
        TouchObject.__init__(self, pos, self.size)
        self.active_color = (0, 150, 255)
        self.normal_box = self.box
        self.active_box = self.font.render(text, True,
                                           self.active_color)

    def update(self):
        return TextItem.update(self)

    def set_text(self, text, change_size):
        TextItem.set_text(self, text, change_size)
        self.normal_box = self.box
        self.active_box = self.font.render(text, True,
                                           self.active_color)

    def set_active(self, active):
        TouchObject.set_active(self, active)
        if self.active:
            self.box = self.active_box
        else:
            self.box = self.normal_box

    def render(self, surface):
        TouchObject.pre_render(self, surface)
        TextItem.render(self, surface)
        TouchObject.post_render(self, surface)


class Progressbar(TouchObject):
    def __init__(self, font, text, pos, size, max_value, value_text):
        BaseItem.__init__(self, pos, size)
        self.value = 0
        self.max = max_value
        self.back_color = (0, 0, 0, 128)
        self.main_color = (0, 150, 255, 150)
        self.surface = pygame.Surface(self.size, pygame.SRCALPHA)\
            .convert_alpha()
        self.surface.fill(self.back_color)
        self.value_text = value_text
        if value_text:
            self.text = TextItem(font, str(max_value), pos, None)
            self.text.pos = (self.pos[0]
                             + self.size[0] / 2 -
                             self.text.size[0] /
                             2, self.text.pos[1])
            self.text.set_text(str(self.value), True)
        else:
            self.text = TextItem(font, text, pos, None)
            self.text.pos = (
                self.pos[0] + self.size[0] / 2 - self.text.size[0] /
                2, self.text.pos[1])

        # Rectangle
        self.rectangle = pygame.Surface(size, pygame.SRCALPHA)\
            .convert_alpha()
        pygame.draw.rect(self.rectangle, (255, 255, 255),
                         self.rectangle.get_rect(),
                         size[1]/20+1)

    def render(self, surface):
        surface.blit(self.surface, self.pos)
        surface.blit(self.rectangle, self.pos)
        self.text.render(surface)

    def set_value(self, value):
        if value != self.value:
            self.value = value
            if self.value_text:
                self.set_text(str(self.value))
                self.text.pos = (self.pos[0]
                                 + self.size[0] / 2 -
                                 self.text.size[0]
                                 / 2, self.text.pos[1])
            self.surface.fill(self.back_color)
            pos_pixel = value * self.size[0] / self.max
            rect = pygame.Rect(0, 0, pos_pixel, self.size[1])
            self.surface.fill(self.main_color, rect)

    def get_pos_value(self, pos):
        x = pos[0] - self.pos[0]
        return x * self.max / self.size[0]

    def set_text(self, text):
        self.text.set_text(text, True)


class ScrollBar(TouchObject):
    def __init__(self, pos, size, max_value, items_on_screen):
        BaseItem.__init__(self, pos,
                          (pos[0] + size[0], pos[1] + size[1]))
        self.pos = pos
        self.size = size
        self.max = max_value
        self.items_on_screen = items_on_screen
        self.current_item = 0
        self.back_bar = pygame.Surface(self.size, pygame.SRCALPHA)\
            .convert_alpha()
        self.back_bar.fill((255, 255, 255, 128))
        self.bar_pos = 0
        if self.max < 1:
            self.bar_size = self.size[1]
        else:
            self.bar_size = math.ceil(
                float(self.items_on_screen) / float(self.max) * float(
                    self.size[1]))
        self.bar = pygame.Surface((self.size[0], self.bar_size)).convert()
        self.bar.fill((255, 255, 255))

    def render(self, surface):
        surface.blit(self.back_bar, self.pos)
        surface.blit(self.bar,
                     (self.pos[0], self.pos[1] + self.bar_pos))

    def touch(self, pos):
        if pos[1] < self.pos[1] + self.bar_pos:
            return -1
        elif pos[1] > self.pos[1] + self.bar_pos + self.bar_size:
            return 1
        else:
            return 0

    def set_item(self, current_item):
        self.current_item = current_item
        self.bar_pos = float(self.current_item) / float(
            self.max) * float(
            self.size[1])
